@@ -289,6 +289,51 @@ pub struct Tag {
 
 impl ApiType for Tag {}
 
+impl Tag {
+    pub fn id(&self) -> u64 {
+        self.id.parse().expect("tag's ID not a number")
+    }
+
+    pub fn tag<'a>(&'a self) -> &'a str {
+        &self.tag
+    }
+
+    pub fn count(&self) -> u64 {
+        self.count.parse().expect("tag's count not a number")
+    }
+
+    pub fn tag_type(&self) -> TagType {
+        use TagType::*;
+        match self.r#type.as_str() {
+            "artist" => Artist,
+            "character" => Character,
+            "copyright" => Copyright,
+            "deprecated" => Deprecated,
+            "metadata" => Metadata,
+            "tag" => Tag,
+            _ => unreachable!("non-standard tag type"),
+        }
+    }
+
+    pub fn ambigious(&self) -> bool {
+        if self.ambiguous == "0" {
+            false
+        } else {
+            true
+        }
+    }
+}
+
+#[derive(Clone, Copy, Debug)]
+pub enum TagType {
+    Artist,
+    Character,
+    Copyright,
+    Deprecated,
+    Metadata,
+    Tag,
+}
+
 #[derive(Clone, Copy, Debug)]
 pub enum Ordering {
     Date,
@@ -341,15 +386,63 @@ impl TagsRequestBuilder {
         self
     }
 
+    /// How tags are sorted.
+    /// _Date, Count, Name._
+    ///
+    /// This is mainly useful with [`send`](#method.send), but ordering works with the other search
+    /// methods as well ([`name`](#method.name), [`names`](#method.names), [`pattern`](#method.pattern))
+    ///
+    /// ## Example
+    /// ```rust
+    /// # async fn example() -> Result<(), ()> {
+    /// # use gelbooru_api::{Client, Ordering, tags};
+    /// # let client = Client::public();
+    /// tags()
+    ///     .limit(5)                 // 5 tags
+    ///     .ascending(true)             // descending
+    ///     .order_by(Ordering::Date) // according to creation-time
+    ///     .send(&client)
+    ///     .await?;
+    /// # Ok(())
+    /// # }
+    /// ```
     pub fn order_by(mut self, ordering: Ordering) -> Self {
         self.order_by = Some(ordering);
         self
     }
 
+    /// Query for tags without name/pattern specifier.
+    ///
+    /// ## Example
+    /// ```rust
+    /// # async fn example() -> Result<(), ()> {
+    /// # use gelbooru_api::{Client, Ordering, tags};
+    /// # let client = Client::public();
+    /// tags()
+    ///     .limit(10)                 // 10 tags
+    ///     .ascending(false)             // descending
+    ///     .order_by(Ordering::Count) // according to count (how many posts have tag)
+    ///     .send(&client)
+    ///     .await?;
+    /// # Ok(())
+    /// # }
+    /// ```
     pub async fn send(self, client: &Client) -> Result<Vec<Tag>, Error> {
         self.search(client, None).await
     }
 
+    /// Pull data for given tag
+    ///
+    /// ## Example
+    /// ```rust
+    /// # async fn example() -> Result<(), ()> {
+    /// # use gelbooru_api::{Client, Ordering, tags};
+    /// # let client = Client::public();
+    /// tags()
+    ///     .name(&client, "solo")
+    ///     .await?;
+    /// # Ok(())
+    /// # }
     pub async fn name<S: AsRef<str>>(self, client: &Client, name: S) -> Result<Option<Tag>, Error> {
         let search = TagSearch::Name(name.as_ref());
         self.search(client, Some(search))
@@ -357,6 +450,20 @@ impl TagsRequestBuilder {
             .map(|tags| tags.into_iter().next())
     }
 
+    /// Pull data for the specified tags
+    ///
+    /// Tag limit is automatically set to accompany all the names.
+    ///
+    /// ## Example
+    /// ```rust
+    /// # async fn example() -> Result<(), ()> {
+    /// # use gelbooru_api::{Client, Ordering, tags};
+    /// # let client = Client::public();
+    /// tags()
+    ///     .names(&client, &["solo", "hatsune_miku"])
+    ///     .await?;
+    /// # Ok(())
+    /// # }
     pub async fn names<S: AsRef<str>>(
         self,
         client: &Client,
@@ -367,6 +474,22 @@ impl TagsRequestBuilder {
         self.search(client, Some(search)).await
     }
 
+    /// Search for tags with a pattern.
+    ///
+    /// Use `_` for single-character wildcards and `%` for multi-character wildcards.
+    /// (`%choolgirl%` would act as `*choolgirl*` wildcard search)
+    ///
+    /// ## Example
+    /// ```rust
+    /// # async fn example() -> Result<(), ()> {
+    /// # use gelbooru_api::{Client, Ordering, tags};
+    /// # let client = Client::public();
+    /// tags()
+    ///     .limit(10)
+    ///     .pattern(&client, "%o_o") // matches regex /.*o.o/
+    ///     .await?;
+    /// # Ok(())
+    /// # }
     pub async fn pattern<S: AsRef<str>>(
         self,
         client: &Client,
@@ -381,9 +504,18 @@ impl TagsRequestBuilder {
         client: &Client,
         search: Option<TagSearch<'_>>,
     ) -> Result<Vec<Tag>, Error> {
+        let limit = self.limit.unwrap_or_else(|| {
+            use TagSearch::*;
+            match &search {
+                Some(Name(_)) => 1,
+                Some(Names(names)) => names.len(),
+                _ => 100,
+            }
+        });
+
         let mut qs: QueryStrings = Default::default();
         qs.insert("s", "tag".to_string());
-        qs.insert("limit", self.limit.unwrap_or(100).to_string());
+        qs.insert("limit", limit.to_string());
 
         if let Some(id) = self.after_id {
             qs.insert("after_id", id.to_string());
@@ -418,6 +550,7 @@ impl TagsRequestBuilder {
     }
 }
 
+// internal function as to DRY
 async fn query_api<T: ApiType>(client: &Client, mut qs: QueryStrings<'_>) -> Result<Vec<T>, Error> {
     if let Some(auth) = &client.auth {
         qs.insert("user_id", auth.user.to_string());
@@ -433,8 +566,6 @@ async fn query_api<T: ApiType>(client: &Client, mut qs: QueryStrings<'_>) -> Res
         .parse::<hyper::Uri>()
         .map_err(|err| Error::UriParse(err))?;
 
-    dbg!(&uri);
-
     let res = client
         .http_client
         .get(uri)
