@@ -6,13 +6,14 @@ async fn main() -> Result<(), Box<dyn std::error::Error>> {
     let auth = AuthDetails::from_query_string(&auth_qs)?;
     let client = Client::with_auth(auth);
 
-    let tags = tags().limit(5).after_id(69420)
-        .ascending(false)
-        .order_by(Ordering::Date)
-        .send(&client).await?;
+    let tags = tags()
+        .limit(10000)
+        .send(&client)
+        .await?;
 
     for tag in tags {
-        dbg!(tag);
+        println!("Real tag: {}", &tag.r#type);
+        println!("Est  tag: {:?}", tag.tag_type());
     }
 
     /*
